# -*- coding: utf-8 -*-
# $Id$

module MakeProj
  #
  # Propertyクラスに含まれる、各オプションやプロパティのためのクラスです。
  # Hashと同じように、キーに対して値のセット、そして追加ができます。
  #
  class Option
    #
    # インスタンスを生成します。
    # parentが与えられたときには、parentの設定を引き継ぎます。
    #
    def initialize(parent = nil)
      @parent = parent
      @hash = Hash.new
    end

    #
    # keyにvalueを設定します。keyは単一の値を持ちます。
    #
    def set(key, value)
      @hash[key] = value
    end

    #
    # keyにvalueを追加します。keyの値は配列になります。
    #
    def append(key, *values)
      option = (@hash[key] ||= Array.new)
      option.concat(values)
    end

    #
    # すべてのキーの配列を返します。親だけが持っているキーも含まれます。
    #
    def keys
      if @parent
        return (@parent.keys + @hash.keys).uniq
      end
      return @hash.keys
    end

    #
    # keyの値を取得します。親の値も引き継いだ値です。
    #
    def get(key)
      if value = @hash[key]
        if @parent && value.is_a?(Array)
          parent_value = @parent.get(key)
          if parent_value.is_a?(Array)
            value = parent_value + value
          end
        end
      else
        if @parent
          value = @parent.get(key)
        end
      end
      return value
    end

    #
    # elementにXMLドキュメントを追加します。
    #
    def add_xml(element)
      keys.each { |key|
        if value = get(key)
          element.add_element(key).
            add_text(value.is_a?(Array) ? value.join(';') : value)
        end
      }
    end
  end
end
