# -*- coding: utf-8 -*-
# $Id$

module MakeProj
  ##
  #  プロジェクト/構成のプロパティ
  #
  class Property
    # 新しいインスタンスを生成します。
    # parentが与えられたときには、それを親としてプロパティを引き継ぎます
    def initialize(parent = nil)
      @properties = Option.new(parent ? parent.properties : nil)
      @compile_options = Option.new(parent ? parent.compile_options : nil)
      @lib_options = Option.new(parent ? parent.lib_options : nil)
      @link_options = Option.new(parent ? parent.link_options : nil)
    end

    # プロパティ(Option)
    attr_reader :properties

    # コンパイルオプション(Option)
    attr_reader :compile_options

    # ライブラリアン(アーカイブ)オプション(Option)
    attr_reader :lib_options
    
    # リンクオプション(Option)
    attr_reader :link_options

    #
    # [全般]-[出力ディレクトリ]を設定します。
    #
    def set_out_dir(dir)
      @properties.set('OutDir', dir)
    end

    #
    # [全般]-[中間ディレクトリ]を設定します。
    #
    def set_int_dir(dir)
      @properties.set('IntDir', dir)
    end

    #
    # [全般]-[ターゲット名]を設定します。
    #
    def set_target_name(name)
      @properties.set('TargetName', name)
    end

    #
    # [C/C++]-[全般]-[追加のインクルードディレクトリ]にディレクトリを追加します。
    #
    def append_include_dir(dir)
      @compile_options.append('AdditionalIncludeDirectories', dir)
    end

    #
    # [C/C++]-[プリプロセッサ]-[プリプロセッサの定義]にマクロを追加します。
    # valueが与えられたときには、"<key>=<value>"の形式になります。
    #
    def append_macro(key, value = nil)
      @compile_options.append('PreprocessorDefinitions', 
                              value ? "#{key}=#{value}" : key)
    end

    #
    # [C/C++]-[全般]-[警告レベル]を設定します(0-4)。
    # 5以上が与えられたときには、/Wallになります。
    #
    def set_warning_level(level)
      if level >= 0 && level <= 4
        @compile_options.set('WarningLevel', "Level#{level}")
      elsif level > 4
        @compile_options.set('WarningLevel', 'EnableAllWarnings')
      else
        raise(Error, 'illegal warning level')
      end
    end

    #
    # [C/C++]-[詳細設定]-[指定の警告を無視する]に追加します。
    #
    def disable_warnings(*warnings)
      @compile_options.append('DisableSpecificWarnings', 
                              *warnings.collect { |warning| warning.to_s })
    end

    #
    # [C/C++]-[全般]-[デバッグ形式の設定]を「C7互換」に設定します。
    # [C/C++]-[コード作成]-[最小リビルドを有効にする]も「いいえ」に設定されます。
    #
    def set_z7
      @compile_options.set('DebugInformationFormat', 'OldStyle')
      @compile_options.set('MinimalRebuild', 'false')
    end

    #
    # [リンカー]-[システム]-[サブシステム]を設定します。
    #
    def set_subsystem(subsystem)
      @link_options.set('SubSystem', subsystem)
    end

    #
    # [リンカー]-[デバッグ]-[デバッグ情報の生成]を「はい」に設定します。
    #
    def set_generate_debug_information
      @link_options.set('GenerateDebugInformation', 'true')
    end

    #
    # [リンカー]-[最適化]-[参照]を「はい」に設定します。
    # [リンカー]-[最適化]-[COMDATの圧縮]も「はい」に設定されます。
    #
    def set_optimize_link
      @link_options.set('OptimizeReferences', 'true')
      @link_options.set('EnableCOMDATFolding', 'true')
    end

    #
    # [リンカー]-[全般]-[追加のライブラリディレクトリ]にディレクトリを追加します。
    #
    def append_lib_dir(*dirs)
      @link_options.append('AdditionalLibraryDirectories', *dirs)
    end

    #
    # [リンカー]-[入力]-[追加の依存ファイル]にライブラリファイルを追加します。
    #
    def append_lib(*libs)
      @link_options.append('AdditionalDependencies', *libs)
    end
  end
end
